<?php
/*
 * CHeMS (Content HElper Management System)
 * Copyright (C) 2007-2008  Claudio M. Alessi
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
*/

/*
 * Categories module
*/

/*
 * The CHeMS module get function
*/
function categories_main()
{
   global $module;
   require(FILE_MODCONF);

   // No multiple instances are allowed for this module
   if( $module->instid )
      return CHEMS_MODULE_FAILURE;

   // Set the current page/post ID
   $id = ($module->modname === "posts_mgr" ?
            ($_GET['post'] ? $_GET['post'] : 0) :
	    ($_GET['page'] ? $_GET['page'] : 0));

   /*
    * Pre-operations depending on the module name
   */

   // Get the file (posts/pages) constant suffix and include the file
   require(@constant("MOD_".($module->modname === "posts_mgr" ? "POST" : "PAGE")."SMGR_DB"));

   // Get the objects list
   $objs = ($module->modname === "posts_mgr" ? $POST : $PAGE);
   $cats = getCats($objs, $id); // Get the categories list

   // Open the divs
   $text = "
      <div class='chems_module'>
         <div class='chems_module_title'>".MOD_CATEGORIES_BLOCKTITLE."</div>
         <div class='chems_module_body'>
   ";

   // Format operators
   $FO = array("%[category_name]", "%[category_elements]");

   // Show all categories
   if( $cats ) {
      
      // Get the object type
      $objtype = ($module->modname === "posts_mgr" ? "post" : "page");

      // Traverse categories.
      // Note. $occur represents the number of occurences for the current category
      foreach($cats as $cat => $occur) {

	 // Format operators expansions
	 $FOEXP = array(
	    "<a href='?modname=categories&amp;name=$cat&amp;objtype=$objtype'>$cat</a>",
	    $occur
	 );

	 // Get the expanded link
	 $link = str_replace($FO, $FOEXP, MOD_CATEGORIES_LINK);

         // Get the zoom value
	 $zoom = ($occur >= MOD_CATEGORIES_ZOOM ? MOD_CATEGORIES_ZOOM : $occur);

	 // If enabled, zoom the categories
	 if( MOD_CATEGORIES_ZOOM )
	    $text .= "<span style='font-size: ".(8 + $zoom)."pt'>$link</span>";
         else
	    $text .= $link; // Just show the link

	 $text .= " "; // Space separator
      }
   }
   else {
      // Don't show the module
      if( MOD_CATEGORIES_UNSHOW )
         return;

      // Show a user message
      $text .= MOD_CATEGORIES_NOCATS;
   }

   // Close the divs
   $text .= "
	 </div>
      </div>
   ";

   // Static format operators
   $FO = array("%[category_type]", "%[categories]");
   $FOEXP = array($objtype, sizeof($cats));

   return str_replace($FO, $FOEXP, $text);
} /* eof categories_main() */

/*
 * The categories page
*/
function categories_page()
{
   require(FILE_MODCONF);

   $cat = $_GET['name'];	// Category name
   $type = $_GET['objtype'];	// Object type

   // Get the file (posts/pages) constant suffix and include the file
   require(@constant("MOD_".strtoupper($type)."SMGR_DB") );

   // Get the objects list
   $objs = ($type === "post" ? $POST : $PAGE);

   // Generate the "expanded" intro
   $cat_text = "
      <div class='chems_module'>
         <div class='chems_module_title'>".MOD_CATEGORIES_PAGETITLE."</div>
         <div class='chems_module_body'>
	    ".MOD_CATEGORIES_INTRO."
   ";

   // The format operators for the MOD_CATEGORIES_OBJECT option
   $FO = array("%[category_object]", "%[category_cats]");

   // Show the objects
   for($o = 1, $occurs = 0; $objs[$o]; $o++) {
      
      // Get the categories list
      $cats = getCats($objs, $o);

      // Check all the categories
      foreach($cats as $key => $val) {

         // If the current object is in the current category ($key), include it
         if( $cat === $key ) {
            $cat_link = "<a href='".WEBROOT."?modname=posts_mgr&amp;post=$o'>".$objs[$o]['title']."</a>";

	    // Links the related categories
	    foreach($cats as $key => $val)
	       $cat_list .= "<a href='".WEBROOT."?modname=categories&amp;name=$key&amp;objtype=$type'>$key</a> "; 
	    ++$occurs;

            // The format operators expansions for the MOD_CATEGORIES_OBJECT option
	    $FOEXP = array($cat_link, $cat_list);

            // Add the current object
	    $cat_text .= str_replace($FO, $FOEXP, MOD_CATEGORIES_OBJECT);

	    $cat_list = ""; // Reset the categories list

	    break;
         }

      }
   }

   // Format operators definitions
   $FO = array(
      "%[category_name]",
      "%[category_elements]",
      "%[category_type]"
   );

   // Format operators expansions
   $FOEXP = array($cat, $occurs, $type);

   $cat_text .= "
         </div>
      </div>
   ";

   return str_replace($FO, $FOEXP, $cat_text);
} /* categories_page() */

/*
 * Get a list of all categories
*/
function getCats(&$objs, $id)
{
   // An ID has been specified, show only a post categories
   if( $id )
      $inf = $sup = $id;

   // No ID specified, merge all categories of the current posts
   else {
      xtrset(sizeof($objs), MOD_POSTS_POSTSPERPAGE, &$inf, &$sup);
   }

   /*
    * Store all categories (only for listing posts).
   */
   for($i = $inf; $i <= $sup; $i++) {
      if( $objs[$i]['categories'] ) {

         // Separate all the elements
         $list = split(FIELDS_SEPARATOR, str_replace(" ", "", $objs[$i]['categories']));

         // Store all the categories
         for($cat = 0; $list[$cat]; $cat++) {
            if( ! $cats[$list[$cat]] )
	       $cats[$list[$cat]] = totCats($objs, $list[$cat]);
         }
      }
   }

   return $cats;
} /* getCats() */

/*
 * Get the totals number of objects which
 * contains the specified category
*/
function totCats(&$objs, $cat)
{
   // Cycle all objects
   for($i = 1, $tot = 0; $objs[$i]; $i++) {
      
      // Get the categories
      $cats = split(FIELDS_SEPARATOR, str_replace(" ", "", $objs[$i]['categories']));

      // Count the occurrences
      foreach($cats as $curcat) {
         if( $cat === $curcat )
	    ++$tot;
      }
   }

   return $tot;
} /* eof totCats() */

?>
